/********************************************************************/
/*  This file contains 3 sections:									*/
/*																	*/
/*	1) A Pre-Init Section - this section saves off all the			*/
/*		registers of the DSP.										*/
/*																	*/
/*	2) A Init Code Section - this section is the customer			*/
/*		initialization code which can be modified by the customer.	*/
/*		As an example, an SDRAM initialization code is supplied.	*/
/*																	*/
/*	3) A Post-Init Section - this section restores all the			*/
/*		register from the stack.									*/
/*																	*/
/*	Customers should not modify the Pre-Init and Post-Init Sections.*/
/*	The Init Code Section can be modified for application use.		*/
/********************************************************************/

#include <defBF533.h>

#define OFFSET_( x ) ((x) & 0x0000FFFF)
#define UPPER_( x ) (((x) >> 16) & 0x0000FFFF)
#define LOWER_( x ) ((x) & 0x0000FFFF)

#define SYSMMR_BASE 0xFFC00000
#define COREMMR_BASE 0xFFE00000

.global _main;
.section program;


_main:
/*******Pre-Init Section*********************************************/
/*******DO NOT MODIFY************************************************/
/*
	[--SP] = ASTAT;
	[--SP] = RETS;
	[--SP] = (r7:0);
	[--SP] = (p5:0)
*/

//loop1:
//	NOP;
//	JUMP loop1;

	[--SP] = ASTAT;			//Save Regs onto stack
	[--SP] = RETS;
	[--SP] = (r7:0);
	[--SP] = (p5:0);
	[--SP] = I0;
	[--SP] = I1;
	[--SP] = I2;
	[--SP] = I3;
	[--SP] = B0;
	[--SP] = B1;
	[--SP] = B2;
	[--SP] = B3;
	[--SP] = M0;
	[--SP] = M1;
	[--SP] = M2;
	[--SP] = M3;
	[--SP] = L0;
	[--SP] = L1;
	[--SP] = L2;
	[--SP] = L3;
	
/********************************************************************/

/*******Init Code Section********************************************/

/**Please insert Initialization code in this section************/
	
	P0.H = UPPER_(COREMMR_BASE);			// P0 Points to the beginning of CORE MMR Space
	P0.L = LOWER_(COREMMR_BASE);

	P1.H = UPPER_(SYSMMR_BASE);				// P1 Points to the beginning of SYSTEM MMR Space
	P1.L = LOWER_(SYSMMR_BASE);

	R0 = 0x10(z);
	W[P1+OFFSET_(SPI_BAUD)] = R0;			// Speed up SPI rate.


// (JWS) Check below if PO or P1 get changed!
/*******SDRAM Setup************/
Setup_SDRAM:
	/* Start PMP */
	
	//*pEBIU_AMBCTL0	= 0x7bb0FbF0;
	P2.H = HI(EBIU_AMBCTL0);
	P2.L = LO(EBIU_AMBCTL0);
	R2.H = HI(0xFFC2FFC2);
	R2.L = LO(0xFFC2FFC2);
	[P2] = R2;
	//*pEBIU_AMBCTL1	= 0x7bb07bb0;
	P2.H = HI(EBIU_AMBCTL1);
	P2.L = LO(EBIU_AMBCTL1);
	R2.H = HI(0xFFC2FFC2);
	R2.L = LO(0xFFC2FFC2);
	[P2] = R2;
	//*pEBIU_AMGCTL	= 0x000f;
	P2.H = HI(EBIU_AMGCTL);
	P2.L = LO(EBIU_AMGCTL);
	R2.H = 0x0000;
	R2.L = 0x000f;
	W[P2] = R2;
	
	//*( (volatile unsigned short *)(0x20100000) ) = 0xFF40;
	// Hold 1.2V core and 1.8V I/O on	
	P2.H = 0x2010;
	P2.L = 0x0000;
	R1.L = 0xFF40; 
	R1.H = 0x0000;
	W[P2] = R1;
	//k = *(volatile unsigned short *)(0x20000000); //dummy read
	P2.H = 0x2000;
	P2.L = 0x0000;
	R1 = W[P2](Z);
	ssync;
	
	//*( (volatile unsigned short *)(0x20100000) ) = 0xFFD0;
	// Also turn on 2.8V rail along with the 5V and 9V rails too.	
	P2.H = 0x2010;
	P2.L = 0x0000;
	R1.L = 0xFFD0; 
	R1.H = 0x0000;
	W[P2] = R1;
	//k = *(volatile unsigned short *)(0x20000000); //dummy read
	P2.H = 0x2000;
	P2.L = 0x0000;
	R1 = W[P2](Z);
	ssync;
	
	//*( (volatile unsigned short *)(0x20100000) ) = 0xFBD0;	
	// Pass SDCKE signal to SDRAM (un-tristate buffer)
	P2.H = 0x2010;
	P2.L = 0x0000;
	R1.L = 0xFBD0; 
	R1.H = 0x0000;
	W[P2] = R1;
	//k = *(volatile unsigned short *)(0x20000000); //dummy read
	P2.H = 0x2000;
 	P2.L = 0x0000;
	R1 = W[P2](Z);
	ssync;
	
	/* Initialize SDRAM registers. */
	P0.L = EBIU_SDRRC & 0xffff;
	P0.H = EBIU_SDRRC >> 16;
	R0.L = 0x0817;								
	W[P0] = R0;
	P0.L = EBIU_SDBCTL & 0xffff;			
	P0.H = EBIU_SDBCTL >> 16;					
	R0.L = 0x0025;		// 0x0011 for Mobile 8megbx16			
	W[P0] = R0.L;
	P0.L = EBIU_SDGCTL & 0xffff;
	P0.H = EBIU_SDGCTL >> 16;
	R0.L = 0x998d;  // was 0x998d        
	R0.H = 0x0491;   //CHANGE FROM 0x0491 TO 0x1491  FOR MOBILE SDRAM !!!!!!!  13ma
	[P0] = R0;
	R3.L = 0xffff;
	R3.H = 0x0000;
	
	// Write out core voltage settings using PF9 and PF10.  PF11 is used to latch value.
	P0.L = FIO_DIR & 0xffff;
	P0.H = FIO_DIR >> 16;
	R0.L = (PF11 | PF10 | PF9 | PF2);								
	W[P0] = R0;
/*	
	// Set PF11 high which sets the SDCKE un-tristated on the external latch
	P0.L = FIO_FLAG_S & 0xffff;
	P0.H = FIO_FLAG_S >> 16;
	R0.L = (PF11 | PF9);								
	W[P0] = R0;
	
	// PF10=0 and PF9=1 is about 1.17V for the core which will give at least 400MHz operation.
	P0.L = FIO_FLAG_C & 0xffff;
	P0.H = FIO_FLAG_C >> 16;
	R0.L = (PF10);								
	W[P0] = R0;
*/
	// Set PF11 high which sets the SDCKE un-tristated on the external latch
	P0.L = FIO_FLAG_S & 0xffff;
	P0.H = FIO_FLAG_S >> 16;
	R0.L = (PF11 | PF2);								
	W[P0] = R0;
	
	// PF10=0 and PF9=0 is about 1.27V for the core which will give at least 400MHz operation.
	P0.L = FIO_FLAG_C & 0xffff;
	P0.H = FIO_FLAG_C >> 16;
	R0.L = (PF9 | PF10);								
	W[P0] = R0;
	
	/* End PMP */
/******************************/
/*
PLL_change:

	P0.L = lo(SIC_IWR);
	P0.H = hi(SIC_IWR);
	R1 = [P0];
	R0 = 0x1; 
	[P0] = R0;
	
	P1.L = lo(PLL_CTL);
	P1.H = hi(PLL_CTL);
	R0.L = 0x3000; 		// Default for BF533 is 0x1400 = 10X CLKIN
				// CLKIN = 12.288MHz for the PMP
				// PLL_CTL = 0[xxx|xxx]0|0000|0000 where xxxxxx is the MSEL value
				// MSEL = 011000 = 24X = 24 X 12.288 = 294.912 MHz
				// PLL_CTL = 0x3000


	w[P1] = R0;
	ssync;
	
	idle;
	
	[P0] = R1;
	
	P1.L = lo(PLL_DIV);
	P1.H = hi(PLL_DIV);
	R0.L = 0x3; 		// Default for BF533 is 0x0005 = CCLK/5
				// CCLK/3 = 294.912/3 = 98.304


	w[P1] = R0;
*/	
/******************************/

/********************************************************************/

/*******Post-Init Section********************************************/

CHECK_KEY:
    P2.H = 0x0000;
    P2.L = 0x3FEC;							//Key is located at address 0x3FEC
    R0.H = 0x1234;
    R0.L = 0x5678;
    R1 = [P2++];
    SSYNC;
    CC = R0 == R1;
    IF CC JUMP RESTORE;						//If the key matches, don't do a full boot

FULL_BOOT:
	L3 = [SP++];
	L2 = [SP++];	
	L1 = [SP++];
	L0 = [SP++];
	M3 = [SP++];
	M2 = [SP++];
	M1 = [SP++];
	M0 = [SP++];
	B3 = [SP++];
	B2 = [SP++];
	B1 = [SP++];
	B0 = [SP++];
	I3 = [SP++];
	I2 = [SP++];
	I1 = [SP++];
	I0 = [SP++];
	(p5:0) = [SP++];  		//Restore Regs from Stack
	(r7:0) = [SP++];
	RETS = [SP++];
	ASTAT = [SP++];
	
/*	(p5:0) = [SP++];		// Hiren did a lot less storage
	(r7:0) = [SP++];		// This may also reduce code size!
	RETS = [SP++];
	ASTAT = [SP++];
	
*/

	RTS;					//If the key doesn't match, return back to the bootrom for a full boot

RESTORE:


/******************************/

PLL_change:

	P0.L = lo(SIC_IWR);
	P0.H = hi(SIC_IWR);
	R1 = [P0];
	R0 = 0x1; 
	[P0] = R0;
	
	P1.L = lo(PLL_CTL);
	P1.H = hi(PLL_CTL);
	R0.L = 0x3000; 		// Default for BF533 is 0x1400 = 10X CLKIN
				// CLKIN = 12.288MHz for the PMP
				// PLL_CTL = 0[xxx|xxx]0|0000|0000 where xxxxxx is the MSEL value
				// MSEL = 011000 = 24X = 24 X 12.288 = 294.912 MHz
				// PLL_CTL = 0x3000


	w[P1] = R0;
	ssync;
	
	idle;
	
	[P0] = R1;
	
	P1.L = lo(PLL_DIV);
	P1.H = hi(PLL_DIV);
	R0.L = 0x3; 		// Default for BF533 is 0x0005 = CCLK/5
				// CCLK/3 = 294.912/3 = 98.304


	w[P1] = R0;
	
/******************************/






	P0.H = UPPER_(COREMMR_BASE);			// P0 Points to the beginning of CORE MMR Space
	P0.L = LOWER_(COREMMR_BASE);

	P1.H = UPPER_(SYSMMR_BASE);				// P1 Points to the beginning of SYSTEM MMR Space
	P1.L = LOWER_(SYSMMR_BASE);
	
//Restore Stack Pointer
	SP = [P2++];
	
//Restore Frame Pointer
	FP = [P2++];

//Restore L1 Memory via DMA
    R0 = [P2++];							//P2 will point to the Return Address after this update
    [P1+OFFSET_(MDMA_S0_CURR_DESC_PTR)] = R0;
    R0 += 0x20;
    [P1+OFFSET_(MDMA_D0_CURR_DESC_PTR)] = R0;
    R0 = (FLOW-0x3000)|(NDSIZE-0x400)|RESTART|WDSIZE_32|DMAEN(z);
    W[P1+OFFSET_(MDMA_S0_CONFIG)] = R0;
    R0 = (FLOW-0x3000)|(NDSIZE-0x400)|RESTART|WDSIZE_32|WNR|DMAEN(z);
    W[P1+OFFSET_(MDMA_D0_CONFIG)] = R0;
    
//Restore Scratchpad Memory via Core reads/writes
	P3.H = 0x0001;							//P3 = SDRAM Memory
	P3.L = 0xC000;
	P4.H = 0xFFB0;							//P4 = Scratchpad Memory
	P4.L = 0x0000;
	P5.H = 0x0000;							//Restore 4096 bytes
	P5.L = 0x0400;
	LSETUP(L3_TO_SCRATCH_begin, L3_TO_SCRATCH_end) LC0 = P5;
L3_TO_SCRATCH_begin:	R0 = [P3++];
L3_TO_SCRATCH_end:		[P4++] = R0;

//Restore EVT Table via Core reads/writes
	P4.H = 0xFFE0;							//P4 = Event Vector Table
	P4.L = 0x2000;
	P5.H = 0x0000;							//Restore 16 entries
	P5.L = 0x0010;
	LSETUP(L3_TO_EVT_begin, L3_TO_EVT_end) LC0 = P5;
L3_TO_EVT_begin:	R0 = [P3++];
L3_TO_EVT_end:		[P4++] = R0;

//Restore IMASK
	P4.H = 0xFFE0;
	P4.L = 0x2104;
	R0 = [P3++];
	[P4] = R0;

//Wait until L1 Memory DMA is complete
TST_RUN_BIT:
    R0 = W[P1+OFFSET_(MDMA_D0_IRQ_STATUS)](z);
    CC = BITTST(R0,3);
    IF CC JUMP TST_RUN_BIT;					//Wait til DMA is complete
 
    
    
   
    
/**** For profiling purposes ****/

//  (JWS) This will probably screw up the SDRAM tristate buffer
//	Unless we or in the other bits
	
/*	R0 = 0x0004;							//Enable PF2 as output
	W[P1+OFFSET_(FIO_DIR)] = R0;
	R0 = 0x0004;							//Set PF2 high
	W[P1+OFFSET_(FIO_FLAG_S)] = R0;


//		*pFIO_FLAG_T = PF2_LED_CS;		// Toggle PF2 LED
	P0.L = FIO_DIR & 0xffff;
	P0.H = FIO_DIR >> 16;
	R0.L = (PF2);								
	W[P0] = R0;
	
	P0.L = FIO_FLAG_T & 0xffff;
	P0.H = FIO_FLAG_T >> 16;
	R0.L = (PF2);								
	W[P0] = R0;
*/

	// Write out core voltage settings using PF9 and PF10.  PF11 is used to latch value.
	P0.L = FIO_DIR & 0xffff;
	P0.H = FIO_DIR >> 16;
	R0.L = (PF11 | PF10 | PF9 | PF2);								
	W[P0] = R0;
	
	// Set PF11 high which sets the SDCKE un-tristated on the external latch
	P0.L = FIO_FLAG_S & 0xffff;
	P0.H = FIO_FLAG_S >> 16;
	R0.L = (PF11);								
	W[P0] = R0;
	
	// PF10=0 and PF9=0 is about 1.27V for the core which will give at least 400MHz operation.
	P0.L = FIO_FLAG_C & 0xffff;
	P0.H = FIO_FLAG_C >> 16;
	R0.L = (PF9 | PF10 | PF2);								
	W[P0] = R0;



//loop1:
//	NOP;
//	JUMP loop1;

		

/********************************/

	P5 = [P2];								// This is the RETS that was saved before going into Hibernation
    JUMP(P5);

    /*

	L3 = [SP++];
	L2 = [SP++];	
	L1 = [SP++];
	L0 = [SP++];
	M3 = [SP++];
	M2 = [SP++];
	M1 = [SP++];
	M0 = [SP++];
	B3 = [SP++];
	B2 = [SP++];
	B1 = [SP++];
	B0 = [SP++];
	I3 = [SP++];
	I2 = [SP++];
	I1 = [SP++];
	I0 = [SP++];
	(p5:0) = [SP++];  		//Restore Regs from Stack
	(r7:0) = [SP++];
	RETS = [SP++];
	ASTAT = [SP++];


	RTS;
	
	*/

_main.end:
